/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.examples;

import convexPairs.Parameters;

public class Deviation implements Parameters {
	private int n_mu;
	private int mup;
	private int mun;
	private int n;
	private int[] lb;
	private int[] ub;
	public String toString() {
		return "Deviation. Sum = "+n_mu+" , mu = "+(double)n_mu/n;
	}
	public Deviation(int n_mu, int n, int[] ub, int[] lb){
		this.n_mu = n_mu;
		mun = (int)Math.round(Math.floor(((double)n_mu)/n));
		mup = (int)Math.round(Math.ceil(((double)n_mu)/n));
		this.n = n;
		
		this.ub = ub;
		this.lb = lb;
	}
	
	public int getN() {
		return n;
	}
	public int evaluateF(int i, int x) {
		return Math.abs(n_mu - n * x);
	}
	
	public int evaluateG(int i, int x) {
		return x;
	}
	
	public int getArgminF(int i) {
//		System.out.println("Beware of silent integer overflow!");
		return n_mu <= n*lb[i] ? lb[i] : 
			n_mu >= n*ub[i] ? ub[i] : 
				n*mup - n_mu < n_mu - n*mun ? mup : mun; 
	}
	public int getDeltaP(int i, int Wi) {
		return Wi >= ub[i] ? Integer.MAX_VALUE :
			Wi == mun && mun != mup ? n*mun + n*mup - 2*n_mu :
				Wi >= mup ? n : -n;
	}
	public int getBPP(int i, int Wi) {
		return Wi >= ub[i] ? 0 :
			Wi == mun && mun != mup ? mup :
				Wi >= mup ? ub[i] : 
					Math.min(ub[i],mun) ;
	}
	
	public int getDeltaN(int i, int Wi) {
		return Wi <= lb[i] ? Integer.MAX_VALUE :
			Wi == mup && mun != mup ? - n*mup - n*mun + 2*n_mu :
				Wi <= mun ? n : -n;
	}
	
	public int getBPN(int i, int Wi) {
		return Wi <= lb[i] ? 0 :
			Wi == mup && mun != mup ? mun :
				Wi <= mun ? lb[i]: 
					 Math.max(lb[i],mup);
	}
	public boolean useGeneralCase(int i) {
		return false; 
	}
	@Override
	public void enforceGLessThanOrEqual(int i, int v) {
	}
	@Override
	public void enforceGGreaterThanOrEqual(int i, int v) {
	}
	@Override
	public void enforceIfGEqualThenFLessThanOrEqual(int i, int v, int u) {
	}
}
