/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.examples;

import java.util.Arrays;

import convexPairs.Parameters;


public class LinearAndMax implements Parameters {
	private int[] a;
	private int V;
	private int[] lx;
	private int[] ux;
	private int[] lxnext;
	private int[] uxprev;
	private boolean[] hasV;
	private boolean[] hasOnlyV;
	private int n;
	public String toString() {
		
		return "Linear And Max. Coefs = "+Arrays.toString(a)+" , count occurences of "+V;
	}
	public LinearAndMax(int[] coef,int V, int[] lb, int[] ub, int[] lb2, int[] ub2, boolean[] hasV, boolean[] hasOnlyV){
		a = coef;
		n= a.length;
		this.V = V;
		this.lx = lb;
		this.ux = ub;
		this.lxnext = lb2;//the second smallest value in the domain.
		this.uxprev = ub2;//the second largest value in the domain.
		this.hasV = hasV;
		this.hasOnlyV = hasOnlyV;
	}
	public int getN() {
		return n;
	}
	public int evaluateF(int i, int x) {
		return a[i] * x;
	}
	public int evaluateG(int i, int x) {
		return x==V?1:0;
	}
	public int getArgminF(int i) {
		return a[i] >= 0 ? lx[i] : ux[i]; 
	}
	
	public int getDeltaP(int i, int Wi) {
		return Wi != V && hasV[i] ?  a[i] * (V - Wi) : Integer.MAX_VALUE;
	}
	public int getBPP(int i, int Wi) {
		return Wi != V && hasV[i]? V : 0;
	}
	public int getDeltaN(int i, int Wi) {
		return Wi == V && ! hasOnlyV[i] ? 
				a[i] >= 0 ? a[i] * (Wi - (lx[i] == V?lxnext[i]:lx[i])) 
						: a[i] * (Wi - (ux[i] ==V?uxprev[i]:ux[i]))
				: Integer.MAX_VALUE;
	}
	public int getBPN(int i, int Wi) {
		return Wi == V && ! hasOnlyV[i] ? 
				a[i] >= 0 ? (lx[i] == V?lxnext[i]:lx[i]) 
						: (ux[i] ==V?uxprev[i]:ux[i])
				: Integer.MAX_VALUE;
	}
	
	public boolean useGeneralCase(int i) {
		return true; 
	}
	@Override
	public void enforceGLessThanOrEqual(int i, int v) {
		//v=0 -> x != V
	}
	@Override
	public void enforceGGreaterThanOrEqual(int i, int v) {
		// v=1 -> x = V
		
	}
	@Override
	public void enforceIfGEqualThenFLessThanOrEqual(int i, int v, int u) {
		// v = 1 -> x = V -> x *a[i] <= u
		// v = 0 -> x != V -> x * a[i] <= u
	}
	
}
