/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.examples;

import java.util.Arrays;

import convexPairs.Parameters;


public class LinearAndSum implements Parameters {
	private int[] a;
	private int[] lx;
	private int[] ux;
	
	public LinearAndSum(int[] coef, int[] ub, int[] lb){
		a = coef;
		this.ux = ub;
		this.lx = lb;
	}
	
	public String toString() {
		
		return "Linear And Sum. Coefs = "+Arrays.toString(a);
	}
	
	public int getN() {
		return a.length;
	}
	public int evaluateF(int i, int x) {
		return a[i] * x;
	}
	public int evaluateG(int i, int x) {
		return x;
	}
	public int getArgminF(int i) {
		return a[i] >= 0 ? lx[i] : ux[i]; 
	}
	public int getDeltaP(int i, int Wi) {
		return Wi < ux[i] ? a[i] : Integer.MAX_VALUE;
	}
	public int getBPP(int i, int Wi) {
		return Wi < ux[i] ? ux[i] : 0;
	}
	public int getDeltaN(int i, int Wi) {
		return Wi > lx[i] ? - a[i] : Integer.MAX_VALUE;
	}
	public int getBPN(int i, int Wi) {
		return Wi > lx[i] ? lx[i] : 0;
	}
	public boolean useGeneralCase(int i) {
		return true;
	}
	public void enforceGLessThanOrEqual(int i, int v) {
		//x <= v
	}
	public void enforceGGreaterThanOrEqual(int i, int v) {
		//x >= v
	}
	public void enforceIfGEqualThenFLessThanOrEqual(int i, int v, int u) {
		//x==v -> a*x <=u
		//i.e.
		//a*v>u -> x != v
	}
}
