/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.inoscar;


import convexPairs.ConvexPairOfSumWithIndices;
import convexPairs.inoscar.FailureException;
import convexPairs.inoscar.ParametersWithOscar;
import oscar.cp.core.*;
import oscar.reversible.*;

public class GenericPropagatorWithIndices extends Constraint {
	private CPVarInt[] X;
	private CPVarInt S1;
	private CPVarInt S2;
	private CPVarInt S3;
	private int s1;
	private int s2;
	private int s3;
	private ParametersWithOscar h;
	private ConvexPairOfSumWithIndices na;
	private ReversibleSetIndexedArray index;
	private ReversiblePointer<Integer> fix1, fix2;
	public GenericPropagatorWithIndices(CPVarInt[] X, ParametersWithOscar h, CPVarInt S1, int s1, CPVarInt S2, int s2, CPVarInt S3, int s3, String name, boolean sort){
		super(X[0].s(),name);
		this.X = X;
		this.S1 = S1;
		this.S2 = S2;
		this.S3 = S3;
		this.s1 = s1;
		this.s2 = s2;
		this.s3 = s3;
		this.h = h;
		index = new ReversibleSetIndexedArray(X[0].s(), 0, X.length-1);
		fix1 = new ReversibleInt(X[0].s(),0);
		fix2 = new ReversibleInt(X[0].s(),0);
		na = new ConvexPairOfSumWithIndices(h,index);
		na.sort = sort;
	}
	
	protected CPOutcome setup(CPPropagStrength l){
        for (int i = 0; i < X.length; i++) {
            if (!X[i].isBound()){
            	if(h.useGeneralCase(i))
            		X[i].callPropagateWhenDomainChanges(this);
            	else
            		X[i].callPropagateWhenBoundsChange(this);
            	X[i].callValBindIdxWhenBind(this, i);
            }else{
            	fix1.setValue(fix1.getValue() + h.evaluateF(i, X[i].value()));
            	fix2.setValue(fix2.getValue() + h.evaluateG(i, X[i].value()));
            	index.removeValue(i);
            }       
        }
        if(S1!=null)S1.callPropagateWhenMaxChanges(this);
        if(S2!=null)S2.callPropagateWhenMinChanges(this);
        if(S3!=null)S3.callPropagateWhenMaxChanges(this);
        return propagate();
	}
	
	protected CPOutcome valBindIdx(CPVarInt x, int i){
		fix1.setValue(fix1.getValue() + h.evaluateF(i, X[i].value()));
    	fix2.setValue(fix2.getValue() + h.evaluateG(i, X[i].value()));
    	index.removeValue(i);
    	return CPOutcome.Suspend;
	}
	protected CPOutcome propagate(){
		h.updateDomains();
		int LB = na.computeLB((S2!=null?S2.getMin():s2)-fix2.getValue(),(S3!=null?S3.getMax():s3)-fix2.getValue());
		
		int b1 = s1-fix1.getValue();
		if(S1!=null){
			if(S1.updateMin(LB+fix1.getValue())==CPOutcome.Failure)return CPOutcome.Failure;
			b1 = S1.getMax()-fix1.getValue();
		}else{
			if(LB > b1) return CPOutcome.Failure;
		}
		
		
		if(S2!=null && S2.updateMax(na.computeU_G(b1)+fix2.getValue())==CPOutcome.Failure)return CPOutcome.Failure;
		if(S3!=null && S3.updateMin(na.computeL_G(b1)+fix2.getValue())==CPOutcome.Failure)return CPOutcome.Failure;
		
		int b2 = s2 - fix2.getValue();
		if(S2!=null) b2 = S2.getMin() - fix2.getValue();
		int b3 = s3 - fix2.getValue();
		if(S3!=null) b3 = S3.getMax() - fix2.getValue();
		try{
			for(int i:index){
				if(h.useGeneralCase(i)){
					na.generalCaseForward(i, b1, b2, b3);
					na.generalCaseBackward(i, b1, b2, b3);
				}else{
					na.specialCaseForward(i, b1, b2, b3);
					na.specialCaseBackward(i, b1, b2, b3);
				}
			}
		}catch(FailureException e){
			return CPOutcome.Failure;
		}
		return CPOutcome.Suspend;
	}
}
