/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.inoscar.examples;

import convexPairs.Parameters;
import convexPairs.Util;
import convexPairs.examples.LinearAndCount;
import convexPairs.inoscar.FailureException;
import convexPairs.inoscar.ParametersWithOscar;

import oscar.cp.core.CPOutcome;
import oscar.cp.core.CPVarInt;

public class LinearAndCountFunction extends ParametersWithOscar{
	private int[] ub, lb, lb2, ub2, a;
	private boolean[] hasV, hasOnlyV;
	private int V;
	private CPVarInt[] X;
	private Parameters f;
	public LinearAndCountFunction(int[] coef, int V, CPVarInt[] X){
		lb = new int[X.length];
		ub = new int[X.length];
		lb2 = new int[X.length];
		ub2 = new int[X.length];
		hasV = new boolean[X.length];
		hasOnlyV = new boolean[X.length];
		a = coef;
		this.V = V;
		f = new LinearAndCount(coef, V, lb,ub,lb2,ub2,hasV,hasOnlyV);
		super.setFunction(f);
		this.X = X;
		updateDomains();
	}
	
	public void updateDomains(){
		for(int i=0;i < X.length;i++){
			ub[i] = X[i].getMax();
			lb[i] = X[i].getMin();
			ub2[i] = X[i].isBound()?0:X[i].valueBefore(ub[i]);
			lb2[i] = X[i].isBound()?0:X[i].valueAfter(lb[i]);
			hasV[i] = X[i].hasValue(V);
			hasOnlyV[i] = hasV[i] && X[i].isBound();
		}
	}
	
	public void enforceGLessThanOrEqual(int i, int v) {
//		System.out.println("x_"+i+" -- "+v);
		if(v==0 && X[i].removeValue(V)==CPOutcome.Failure){
			throw new FailureException();
		}
	}
	
	public void enforceGGreaterThanOrEqual(int i, int v) {
//		System.out.println("x_"+i+" -- "+v);
		if(v==1 && X[i].assign(V)==CPOutcome.Failure){
			throw new FailureException();
		}
		
	}
	
	public void enforceIfGEqualThenFLessThanOrEqual(int i, int v, int t) {
//		System.out.println(Arrays.toString(X));
//		System.out.println(Arrays.toString(a));
//		System.out.println(i+"\t"+v+"\t"+t);
		if(v==1){
			if(f.evaluateF(i, V) > t){
				if(X[i].removeValue(V)==CPOutcome.Failure) throw new FailureException();
			}
		}else if(v==0){
			if(f.evaluateF(i,V) <= t || !X[i].hasValue(V)){
//				if(a[i]!=0)System.out.println("here "+a[i]+" "+(t/a[i])+" "+X[i]);
				if(a[i] > 0 && X[i].updateMax(Util.divFloor(t,a[i]))==CPOutcome.Failure) throw new FailureException();
				else if(a[i] < 0 && X[i].updateMin(Util.divCeil(t,a[i]))==CPOutcome.Failure) throw new FailureException();
				else if(a[i]==0 && 0 > t) throw new FailureException();
			}else{
				if(a[i] > 0){
					if(X[i].updateMax(V)==CPOutcome.Failure) throw new FailureException();
					int tmp = Util.divFloor(t,a[i]);
					for(int k=V-1;k > tmp;k--)
						if(X[i].removeValue(k)==CPOutcome.Failure) throw new FailureException();
				}else if (a[i] < 0){
					if(X[i].updateMin(V)==CPOutcome.Failure) throw new FailureException();
					int tmp = Util.divCeil(t,a[i]);
					for(int k=V+1;k < tmp;k++)
						if(X[i].removeValue(k)==CPOutcome.Failure) throw new FailureException();
				}else if(a[i]==0 && 0 > t) throw new FailureException();
			}
		}
	}

}
