/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.inoscar.examples;

import convexPairs.Parameters;
import convexPairs.examples.SumAloneC2;
import convexPairs.inoscar.FailureException;
import convexPairs.inoscar.ParametersWithOscar;
import oscar.cp.core.CPOutcome;
import oscar.cp.core.CPVarInt;

public class SumAloneC2Function extends ParametersWithOscar{
	private int[] ub, lb;
	private CPVarInt[] X;
	private Parameters f;
	public SumAloneC2Function(CPVarInt[] X){
		ub = new int[X.length];
		lb = new int[X.length];
		f = new SumAloneC2(X.length, ub,lb);
		super.setFunction(f);
		this.X = X;
		updateDomains();
	}
	
	public void updateDomains(){
		for(int i=0;i < X.length;i++){
			ub[i] = X[i].getMax();
			lb[i] = X[i].getMin();
		}
	}
	
	public void enforceGLessThanOrEqual(int i, int v) {
		if(X[i].updateMax(v)==CPOutcome.Failure){
			throw new FailureException();
		}
	}
	
	public void enforceGGreaterThanOrEqual(int i, int v) {
		if(X[i].updateMin(v)==CPOutcome.Failure){
			throw new FailureException();
		}	
	}
	
	public void enforceIfGEqualThenFLessThanOrEqual(int i, int v, int t) {
		//x=v -> 0 <= u
		if(t>0){
			System.out.println("bizarre "+t);
			if(X[i].removeValue(v)==CPOutcome.Failure) throw new FailureException();
		}else{
			//
		}
	}

}
