/*
 * Main author(s) : Jean-Noël Monette
 * 
 * Copyright (c) 2013, Jean-Noël Monette
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package convexPairs.inoscar.tests

import oscar.cp.modeling._
import oscar.cp.core._
import scala.Iterable
import convexPairs.inoscar.modeling.Constraints

object TestFullSearchSumAlone extends App with Constraints {
     var S1 = List[String]();
     var S2 = List[String]();
      def testOrig(n:Int, m:Int):Int = {
      	S1 = List[String]();
      	val cp = CPSolver(); 
      	val s1 = CPVarInt(cp,0 to m);
      	val s2 = CPVarInt(cp,0 to m);
      	val rng = 0 until n;
      	val x = for(i <- rng) yield CPVarInt(cp,-5 to 5);

      	val t1 = System.currentTimeMillis();
      	
      	var nbsol = 0
      	cp.solve subjectTo {
      		cp.add(sum(rng)(i => x(i)) == s2);
      		println(x.mkString(","));
      		println("s2 = "+s2)
      	} exploration {        
      	cp.binary(x);
        nbsol += 1;
        S1 = (x.mkString(",") :: S1);
      }run()
      	  cp.printStats()
        
        val t2 = System.currentTimeMillis();
        print("orig: "+(t2-t1)+"\t");
        print("bkts: "+cp.bkts+"\t");
        return nbsol;
      	}
      
      def testNew(n:Int, m:Int):Int = {	
      	S2 = List[String]();
      	val cp = CPSolver(); 
      	val s1 = CPVarInt(cp,0 to m);
      	val s2 = CPVarInt(cp,0 to m);
      	val rng = 0 until n;
      	val x = for(i <- rng) yield CPVarInt(cp,-5 to 5);

      	val t1 = System.currentTimeMillis();
      	
        var nbsol = 0
      	cp.solve subjectTo {
      		cp.add(sumMy(x, s2));
      		//cp.add(linearAndSum2(x, s1, s2, a));
      		println(x.mkString(","));
      		println("s2 = "+s2)
      	} exploration {
      	cp.binary(x);
        S2 = (x.mkString(",") :: S2);
        nbsol += 1
      }run()
      	  cp.printStats()
      	val t2 = System.currentTimeMillis();
        print("new: "+(t2-t1)+"\t");
        print("bkts: "+cp.bkts+"\t");
        return nbsol
      	}
        def testNew2(n:Int, m:Int):Int = {	
      	S2 = List[String]();
      	val cp = CPSolver(); 
      	val s1 = CPVarInt(cp,0 to m);
      	val s2 = CPVarInt(cp,0 to m);
      	val rng = 0 until n;
      	val x = for(i <- rng) yield CPVarInt(cp,-5 to 5);

      	val t1 = System.currentTimeMillis();
      	
        var nbsol = 0
      	cp.solve subjectTo {
      		cp.add(sumMy2(x, s2));
      		println(x.mkString(","));
      		println("s2 = "+s2)
      	} exploration {
      	cp.binary(x);
        S2 = (x.mkString(",") :: S2);
        nbsol += 1
      }run()
      	  cp.printStats()
      	val t2 = System.currentTimeMillis();
        print("new2: "+(t2-t1)+"\t");
        print("bkts: "+cp.bkts+"\t");
        return nbsol
      	}  
      var i=3;
      var j=1;
      while(j< 10 && i<10){
      	print(i+"\t");
      	print(j+"\t");
      	//val a = for(i <- 0 until i) yield i*i; 
    	  val n0 = testOrig(i,j);
    	  val n1 = testNew(i,j);
    	  val n2 = testNew2(i,j);
    	  i=i+1;
    	  //j=j+1;

    	  println(n0 +" "+n1+ " "+ n2);
    	  println();
    	  
    	  if(n0 != n1){
    	  	print(S2.filter(x => !S1.contains(x)).mkString("\n"));
    		  print("PROBLEM ");
    		  System.exit(0);
    	  }    	  
      }
}